"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.wrap = exports.KEY_CFG_METHOD_PREFIX = exports.KEY_CFG_SUFFIX = exports.KEY_CFG_DEFAULT = exports.METADATA_ORDER_LOWEST = exports.KEY_METADATA_DATA_PROCESSORS = exports.KEY_METADATA_URL_PROCESSORS = exports.KEY_METADATA_ORDER = exports.KEY_METADATA = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _Comb = require("./util/Comb");

var Comb = _interopRequireWildcard(_Comb);

var _index = require("./requestor/index");

var _index2 = require("./util/index");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var METHOD_NAMES = ['GET', 'POST', 'PUT', 'DELETE'];

/**
 * config metadata key
 * @type {string}
 */
var KEY_METADATA = exports.KEY_METADATA = '_';

/**
 * metadata order key
 * @type {string}
 */
var KEY_METADATA_ORDER = exports.KEY_METADATA_ORDER = 'order';

/**
 * url processors key
 * @type {string}
 */
var KEY_METADATA_URL_PROCESSORS = exports.KEY_METADATA_URL_PROCESSORS = 'urlProcessors';

/**
 * data processors key
 * @type {string}
 */
var KEY_METADATA_DATA_PROCESSORS = exports.KEY_METADATA_DATA_PROCESSORS = 'dataProcessors';

/**
 * lowest order value which is the last key
 * @type {number}
 */
var METADATA_ORDER_LOWEST = exports.METADATA_ORDER_LOWEST = 9999;

/**
 * default for config
 * @type {string}
 */
var KEY_CFG_DEFAULT = exports.KEY_CFG_DEFAULT = 'defConfig';

/**
 * suffix configs
 * @type {string}
 */
var KEY_CFG_SUFFIX = exports.KEY_CFG_SUFFIX = 'sufConfigs';

/**
 * method prefix, default to 'json'
 * @type {string}
 */
var KEY_CFG_METHOD_PREFIX = exports.KEY_CFG_METHOD_PREFIX = 'methodPrefix';

var wrap = exports.wrap = function wrap(config) {
    var defConfig = config[KEY_CFG_DEFAULT] || {};
    var sufConfigs = config[KEY_CFG_SUFFIX] || {};
    var methodPrefix = config[KEY_CFG_METHOD_PREFIX] || 'json';

    var getConfigOrder = function getConfigOrder(configKey) {

        if (sufConfigs[configKey] && sufConfigs[configKey][KEY_METADATA]) {
            return sufConfigs[configKey][KEY_METADATA][KEY_METADATA_ORDER] || METADATA_ORDER_LOWEST;
        }
        return METADATA_ORDER_LOWEST;
    };

    var sortedSufConfigKeys = Object.keys(sufConfigs).sort(function (configA, configB) {
        var orderA = getConfigOrder(configA);
        var orderB = getConfigOrder(configB);

        return orderA - orderB;
    });

    return METHOD_NAMES.reduce(function (fns, method) {
        return Comb.power(sortedSufConfigKeys).reduce(function (fs, sfs) {
            return _extends({}, fs, _defineProperty({}, "" + methodPrefix + method + sfs.join(''), factory(method, getConfigForSufs(sfs, sufConfigs, defConfig))));
        }, fns);
    }, {});
};

function factory(method, opts) {
    opts = opts || {};

    var mergedConfig = opts[KEY_METADATA] || {};
    delete opts[KEY_METADATA];

    return function (url, data, options) {

        url = getUrl(url, mergedConfig);

        data = getData(data, mergedConfig);

        return (0, _index.doRequest)(method, opts, mergedConfig, url, data, options).then(function (data) {
            if (data instanceof String) {
                data = JSON.parse(data);
            }

            return data;
        });
    };
}

function getUrl(url, mergedConfig) {

    var urlProcessors = mergedConfig[KEY_METADATA_URL_PROCESSORS];
    if (urlProcessors) {
        return Object.keys(urlProcessors).reduce(function (url, processorName) {
            return urlProcessors[processorName](url);
        }, url);
    }

    return url;
}

function getData(data, mergedConfig) {

    var processors = mergedConfig[KEY_METADATA_DATA_PROCESSORS];
    if (processors) {
        return Object.keys(processors).reduce(function (data, processorName) {
            return processors[processorName](data);
        }, data);
    }

    return data;
}

var getConfigForSufs = function getConfigForSufs(sfs, sufConfigs, defConfig) {

    return sfs.reduce(function (result, suf) {
        return (0, _index2.extend)(true, {}, result, sufConfigs[suf]);
    }, _extends({}, defConfig));
};